#include "../include/font_writer.hpp"
#include "FontWriter/font_writer.hpp"
#include "Objects/paco.hpp"
#include "Overlay/Overlay.hpp"
#include "assets.hpp"
#include <PSX/GPU/gpu.hpp>
#include <PSX/GPU/make_gpu_primitives.hpp>
#include <PSX/Periphery/periphery.hpp>
#include <PSX/Timer/frame_timer.hpp>
#include <stdio.h>

using namespace JabyEngine;
using NewFontWriter = ::JabyEngine::FontWriter;

static object::Paco  paco;

static void setup() {
    font_writer_setup();
    Assets::load_for_main();
    FontWriter::FontWriter::setup();

    paco.setup();
}

static void update() {
    Periphery::query_controller();
    
    FontWriter::FontWriter cursor;

    const auto end_pos = cursor.write(FontWriter::Position::create(0, 32), "Cody is cute\n&\na \x1b[8;0;0mBAAAAABY!!!");
    cursor.write(end_pos, "\x1b[0;7;7mJaby was\nhere c:");

    font_writer_update();
    paco.update();
}

static void render() {
    GPU::swap_buffers_vsync(1);
    FontWriter::FontWriter::render();
    paco.render();
    font_writer_render();
}

void main() {
    setup();

    Overlay::TimerTest::mesaure_busy_loop();
    JabyEngine::HighResTime::enable();

    while(true) {
        const auto start = Overlay::TimerTest::start_measuring();
            update();
            render();
        Overlay::TimerTest::end_measuring(start, GPU::Display::frames_per_sec);
    }
}